<?php
class db
{

private static $conn;
private static $stats;
private static $emode;
private static $exname;

private static $defaults = array(
'host' => 'localhost',
'user' => '',
'pass' => '',
'db' => '',
'port' => NULL,
'socket' => NULL,
'pconnect' => FALSE,
'charset' => 'cp1251',
'errmode' => 'error', //or exception
'exception' => 'Exception', //Exception class name
);

const RESULT_ASSOC = MYSQLI_ASSOC;
const RESULT_NUM = MYSQLI_NUM;

public static function connect($opt = array())
{
$opt = array_merge(self::$defaults,$opt);

self::$emode = $opt['errmode'];
self::$exname = $opt['exception'];

if ($opt['pconnect'])
{
$opt['host'] = "p:".$opt['host'];
}

@self::$conn = mysqli_connect($opt['host'], $opt['user'], $opt['pass'], $opt['db'], $opt['port'], $opt['socket']);
if ( !self::$conn )
{
self::error(mysqli_connect_errno()." ".mysqli_connect_error());
}

mysqli_set_charset(self::$conn, $opt['charset']) or self::error(mysqli_error(self::$conn));
unset($opt);
}
public static function query()
{	
return self::rawQuery(self::prepareQuery(func_get_args()));
}
public static function fetch($result,$mode=self::RESULT_ASSOC)
{
return mysqli_fetch_array($result, $mode);
}
public static function affectedRows()
{
return mysqli_affected_rows (self::$conn);
}
public static function insertId()
{
return mysqli_insert_id(self::$conn);
}
public static function numRows($result)
{
return mysqli_num_rows($result);
}
public static function free($result)
{
mysqli_free_result($result);
}
public static function getOne()
{
$query = self::prepareQuery(func_get_args());
if ($res = self::rawQuery($query))
{
$row = self::fetch($res);
if (is_array($row)) {
return reset($row);
}
self::free($res);
}
return FALSE;
}
public static function getRow()
{
$query = self::prepareQuery(func_get_args());
if ($res = self::rawQuery($query)) {
$ret = self::fetch($res);
self::free($res);
return $ret;
}
return FALSE;
}
public static function getCol()
{
$ret = array();
$query = self::prepareQuery(func_get_args());
if ( $res = self::rawQuery($query) )
{
while($row = self::fetch($res))
{
$ret[] = reset($row);
}
self::free($res);
}
return $ret;
}
public static function getAll()
{
$ret = array();
$query = self::prepareQuery(func_get_args());
if ( $res = self::rawQuery($query) )
{
while($row = self::fetch($res))
{
$ret[] = $row;
}
self::free($res);
}
return $ret;
}
public static function getInd()
{
$args = func_get_args();

$index = array_shift($args);
$query = self::prepareQuery($args);



$ret = array();
if ( $res = self::rawQuery($query) )
{
while($row = self::fetch($res))
{
$ret[$row[$index]] = $row;
}
self::free($res);
}
return $ret;
}
public static function getIndCol()
{
$args = func_get_args();
$index = array_shift($args);
$query = self::prepareQuery($args);

$ret = array();
if ( $res = self::rawQuery($query) )
{
while($row = self::fetch($res))
{
$key = $row[$index];
unset($row[$index]);
$ret[$key] = reset($row);
}
self::free($res);
}
return $ret;
}
public static function parse()
{
return self::prepareQuery(func_get_args());
}
public static function whiteList($input,$allowed,$default=FALSE)
{
$found = array_search($input,$allowed);
return ($found === FALSE) ? $default : $allowed[$found];
}
public static function filterArray($input,$allowed)
{
foreach(array_keys($input) as $key )
{
if ( !in_array($key,$allowed) )
{
unset($input[$key]);
}
}
return $input;
}
public static function lastQuery()
{
$last = end(self::$stats);
return $last['query'];
}
public static function getStats()
{
return self::$stats;
}
private static function rawQuery($query)
{
$start = microtime(TRUE);
$res = mysqli_query(self::$conn, $query);
$timer = microtime(TRUE) - $start;

self::$stats[] = array(
'query' => $query,
'start' => $start,
'timer' => $timer,
);
if (!$res)
{
$error = mysqli_error(self::$conn);

end(self::$stats);
$key = key(self::$stats);
self::$stats[$key]['error'] = $error;
self::cutStats();

self::error("$error. Full query: [$query]");
}
self::cutStats();
return $res;
}

private static function prepareQuery($args)
{
$query = '';
$raw = array_shift($args);
$array = preg_split('~(\?[nsiuap])~u',$raw,null,PREG_SPLIT_DELIM_CAPTURE);
$anum = count($args);
$pnum = floor(count($array) / 2);
if ( $pnum != $anum )
{
self::error("Number of args ($anum) doesn't match number of placeholders ($pnum) in [$raw]");
}

foreach ($array as $i => $part)
{
if ( ($i % 2) == 0 )
{
$query .= $part;
continue;
}

$value = array_shift($args);
switch ($part)
{
case '?n':
$part = self::escapeIdent($value);
break;
case '?s':
$part = self::escapeString($value);
break;
case '?i':
$part = self::escapeInt($value);
break;
case '?a':
$part = self::createIN($value);
break;
case '?u':
$part = self::createSET($value);
break;
case '?p':
$part = $value;
break;
}
$query .= $part;
}
return $query;
}

private static function escapeInt($value)
{
if ($value === NULL)
{
return 'NULL';
}
if(!is_numeric($value))
{
//self::error("Integer (?i) placeholder expects numeric value, ".gettype($value)." given");
//return FALSE;
$value = intval($value);
}
if (is_float($value))
{
$value = number_format($value, 0, '.', ''); // may lose precision on big numbers
}
return $value;
}

private static function escapeString($value)
{
if ($value === NULL)
{
return 'NULL';
}
return	"'".mysqli_real_escape_string(self::$conn,$value)."'";
}

private static function escapeIdent($value)
{
if ($value)
{
return "`".str_replace("`","``",$value)."`";
} else {
self::error("Empty value for identifier (?n) placeholder");
}
}

private static function createIN($data)
{
if (!is_array($data))
{
self::error("Value for IN (?a) placeholder should be array");
return;
}
if (!$data)
{
return 'NULL';
}
$query = $comma = '';
foreach ($data as $value)
{
$query .= $comma.self::escapeString($value);
$comma = ",";
}
return $query;
}

private static function createSET($data)
{
if (!is_array($data))
{
self::error("SET (?u) placeholder expects array, ".gettype($data)." given");
return;
}
if (!$data)
{
self::error("Empty array for SET (?u) placeholder");
return;
}
$query = $comma = '';
foreach ($data as $key => $value)
{
$query .= $comma.self::escapeIdent($key).'='.self::escapeString($value);
$comma = ",";
}
return $query;
}

private static function error($err)
{
$err = __CLASS__.": ".$err;

if ( self::$emode == 'error' )
{
$err .= ". Error initiated in ".self::caller().", thrown";
trigger_error($err,E_USER_ERROR);
} else {
throw new self::$exname($err);
}
}

private static function caller()
{
$trace = debug_backtrace();
$caller = '';
foreach ($trace as $t)
{
if ( isset($t['class']) && $t['class'] == __CLASS__ )
{
$caller = $t['file']." on line ".$t['line'];
} else {
break;
}
}
return $caller;
}
private static function cutStats()
{
if ( count(self::$stats) > 100 )
{
reset(self::$stats);
$first = key(self::$stats);
unset(self::$stats[$first]);
}
}
}